#clear the memory
rm(list=ls())

##################################
# DATA IMPORTATION - DESCRIPTION #
##################################

#modifying the default working directory
setwd("D:/_Travaux/university/Cours_Universite/Supports_de_cours/Informatique/R/Tutoriels/cah_kmeans")

#loading the dataset - options are essential
fromage <- read.table(file="fromage.txt",header=T,row.names=1,sep="\t",dec=".")

#displaying the first data rows
print(head(fromage))

#summary - descriptive statistics
print(summary(fromage))

#pairwise scatterplots
pairs(fromage)

#############################################
# HAC - Hierarchical Agglomerative Clustering
#############################################

#standardizing the variables
fromage.cr <- scale(fromage,center=T,scale=T)

#pairwise distance matrix
d.fromage <- dist(fromage.cr)

#HAC  Ward method
cah.ward <- hclust(d.fromage,method="ward.D2")

#plotting the dendrogram
plot(cah.ward)

#dendrogram with highlighting of the groups
rect.hclust(cah.ward,k=4)

#partition in 4 groups
groupes.cah <- cutree(cah.ward,k=4)

#assignment of the instances to clusters
print(sort(groupes.cah))

######################################
# K-MEANS
######################################

#k-means from the standardized variables
#center = 4 - number of clusters
#nstart = 5 - number of trials with different starting centroids
groupes.kmeans <- kmeans(fromage.cr,centers=4,nstart=5)

#displaying the results
print(groupes.kmeans)

#crosstabs with the clusters coming from HAC
print(table(groupes.cah,groupes.kmeans$cluster))

######################################
# K-MEANS - DETECTION NB. DE GROUPES
######################################

#(1)elbow method
# growth of proportion of variance explained according to the number of clusters
inertie.expl <- rep(0,times=10)
for (k in 2:10){
  clus <- kmeans(fromage.cr,centers=k,nstart=5)  
  inertie.expl[k] <- clus$betweenss/clus$totss
}

#plotting
plot(1:10,inertie.expl,type="b",xlab="Nb. de groupes",ylab="% inertie explique")

#(2) fpc package 
library(fpc)

#values of the criterion according to the number of clusters
sol.kmeans <- kmeansruns(fromage.cr,krange=2:10,criterion="ch")

#plotting
plot(1:10,sol.kmeans$crit,type="b",xlab="Nb. of groups",ylab="Calinski-Harabasz")

###############################################
# INTERPRETATION - COND. DESCRIPTIVE STATISTICS
###############################################

#Function for calculating summary statistics
#x characterization variable, y group membership variable
stat.comp <- function(x,y){
  #number of clusters
  K <- length(unique(y))
  #nb. Of instances
  n <- length(x)
  #overall mean
  m <- mean(x)
  #total sum of squares
  TSS <- sum((x-m)^2)
  #size of clusters
  nk <- table(y)
  #conditional mean
  mk <- tapply(x,y,mean)
  #between (explained) sum of squares
  BSS <- sum(nk * (mk - m)^2)
  #collect in a vector the means and the proportion of variance explained
  result <- c(mk,100.0*BSS/TSS)
  #set a name to the values
  names(result) <- c(paste("G",1:K),"% epl.")
  #return the results
  return(result)
}

#applying the function to the original variables of the dataset
#and not to the standardized variables
print(sapply(fromage,stat.comp,y=groupes.cah))

######################################
# HAC and PCA
######################################


#PCA
acp <- princomp(fromage,cor=T,scores=T)

#scree plot  Retain the two first factors
plot(1:9,acp$sdev^2,type="b",xlab="Nb. de factors",ylab="Eigen Val.")

#biplot
biplot(acp,cex=0.65)

#highlight the clusters into the individuals factor map of PCA
plot(acp$scores[,1],acp$scores[,2],type="n",xlim=c(-5,5),ylim=c(-5,5))
text(acp$scores[,1],acp$scores[,2],col=c("red","green","blue","black")[groupes.cah],cex=0.65,labels=rownames(fromage),xlim=c(-5,5),ylim=c(-5,5))

######################################
# RETRAIT DES FROMAGES FRAIS
######################################

#remove the instance corresponding to the 4th group
fromage.subset <- fromage[groupes.cah!=4,]

#standardizing again the dataset
fromage.subset.cr <- scale(fromage.subset,center=T,scale=T)

#distance matrix
d.subset <- dist(fromage.subset.cr)

#HAC  2nd version
cah.subset <- hclust(d.subset,method="ward.D2")

#displaying the dendrogram
plot(cah.subset)

#partitioning into 3 groups
groupes.subset <- cutree(cah.subset,k=3)

#displaying the group membership for each case
print(sort(groupes.subset))

#pca
acp.subset <- princomp(fromage.subset,cor=T,scores=T)

#scree plot
plot(1:9,acp.subset$sdev^2,type="b")

#biplot
biplot(acp.subset,cex=0.65)

#scatter plot - individuals factor map
plot(acp.subset$scores[,1],acp.subset$scores[,2],type="n",xlim=c(-6,6),ylim=c(-6,6))

#row names + group membership
text(acp.subset$scores[,1],acp.subset$scores[,2],col=c("red","green","blue")[groupes.subset],cex=0.65,labels=rownames(fromage.subset),xlim=c(-6,6),ylim=c(-6,6))

#conditional stat.
print(sapply(fromage.subset,stat.comp,y=groupes.subset))
